/*
 * Copyright (C) 2002-2011 OSS Nokalva.  All rights reserved.
 *
 */

/*
 * THIS FILE IS PROPRIETARY MATERIAL OF OSS NOKALVA, INC. AND MAY BE USED
 * ONLY BY DIRECT LICENSEES OF OSS NOKALVA, INC. THIS FILE MAY NOT BE
 * DISTRIBUTED.
 */

/*  
 * This test shows the process of safe PDU relaying in the situation when
 * the sender application has a more recent version of a protocol with all
 * the extension additions known.  The receiver has an older version of the
 * same protocol which is not aware of the extension additions sent.  The
 * receiver gets the unknown extension additions in the decoded PDU, changes 
 * the PDU as needed based on the version of the protocol it works with 
 * (modification not shown in this example), re-encodes the decoded PDU using
 * its older version of the protocol and sends it back to the receiver 
 * application or to another receiver (relays) who is aware of the extension 
 * additions and who succesfully decodes the PDU which was encoded by the sender
 * with the older version of the protocol.  This test is using the ALIGNED 
 * variant of PER but it can be run using BER/DER/UPER as well.  The following
 * ASN.1 compiler command line was used:
 *
 * asn1 -relay relay.asn -per -ber -uper -der -nounique
 */


#include "relay.h"


void *pdus[] = {
    &pdu2,
    &pdu4,
    &pdu6
};

int pdu_num[] = {
    SET1_PDU,
    SET2_PDU,
    SET3_PDU,
    SET4_PDU,
    SET5_PDU,
    SET6_PDU
};


char *names[] = {
    "SET1",
    "SET2",
    "SET3",
    "SET4",
    "SET5",
    "SET6"
};





int main(void)
{

    int         i, j, rc;
    OssGlobal   ww, *world = &ww;
    OssBuf      ossbuf;
    void       *decoded;
    

    if ((rc = ossinit(world, relay)) != 0) {
	printf("Initialization failed with error code =  %d.\n", rc);
	return 1;
    }

    ossSetFlags(world, DEBUGPDU);
    ossSetEncodingRules(world, OSS_PER_ALIGNED);

    for (i = 0, j = 1; i < 3; i++, j += 2) {
	ossPrint(world, "\tUnencoded %s PDU with all the extensions known...\n\n", names[j]);
	ossPrintPDU(world, pdu_num[j], pdus[i]);
	ossbuf.value = NULL;
	ossbuf.length = 0;
	ossPrint(world, "\n\tEncoding %s PDU...\n\n", names[j]);
	rc = ossEncode(world, pdu_num[j], pdus[i], &ossbuf);
	if (rc) {
	    ossPrint(world, "Error encoding.\n");
	    ossPrint(world, ossGetErrMsg(world));
	    return 1;
	}
	else {
	    ossPrint(world, "\n\tEncoded %s PDU with all the extensions known...\n\n", names[j]);
	    ossPrintHex(world, (char *)ossbuf.value, ossbuf.length);
	}

	decoded = NULL;
	ossPrint(world, "\n\tDecoding the encoded %s PDU using %s PDU...\n\n", names[j], names[j - 1]);
	rc = ossDecode(world, &pdu_num[j - 1], &ossbuf, (void **)&decoded);
	if (rc) {
	    ossPrint(world, "Error decoding.\n");
	    ossPrint(world, ossGetErrMsg(world));
	    return 1;
	}
	else {
	    ossPrint(world, "\n\tDecoded PDU...\n\n");
	    ossFreeBuf(world, ossbuf.value);
	    ossPrintPDU(world, pdu_num[j - 1], decoded);
	}

	ossPrint(world, "\n\tRe-encoding the decoded %s PDU with unknown extensions...\n\n", names[j - 1]);
	ossbuf.value = NULL;
	ossbuf.length = 0;
	rc = ossEncode(world, pdu_num[j - 1], decoded, &ossbuf);
	if (rc) {
	    ossPrint(world, "Error encoding.\n");
	    ossPrint(world, ossGetErrMsg(world));
	    return 1;
	}
	else {
	    ossFreePDU(world, pdu_num[j - 1], decoded);
	    ossPrint(world, "\n\tEncoded %s PDU with all the extensions known...\n\n", names[j]);
	    ossPrintHex(world, (char *)ossbuf.value, ossbuf.length);
	}

	decoded = NULL;
	ossPrint(world, "\n\tDecoding the relayed %s PDU...\n\n", names[j]);
	rc = ossDecode(world, &pdu_num[j], &ossbuf, (void **)&decoded);
	if (rc) {
	    ossPrint(world, "Error decoding.\n");
	    ossPrint(world, ossGetErrMsg(world));
	    return 1;
	}
	else {
	    ossPrint(world, "\n\tDecoded relayed %s PDU...\n\n", names[j]);
	    ossFreeBuf(world, ossbuf.value);
	    ossPrintPDU(world, pdu_num[j], decoded);
	    ossFreePDU(world, pdu_num[j], decoded);
	}
    }

    ossterm(world);
    return 0;
}

